@@ -1,30 +1,23 @@
-## Copyright (C) 2008 Sren Hauberg
-## 
-## This program is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or
-## (at your option) any later version.
-## 
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-## 
-## You should have received a copy of the GNU General Public License
-## along with this program; if not, write to the Free Software
-## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+## Copyright (C) 1999 Andy Adler <adler@sce.carleton.ca>
+## Copyright (C) 2008 Søren Hauberg <soren@hauberg.org>
 ##
-## Note: The implementation and help text for the 'andy' edge detector came with
-## the following notice:
-##   Copyright (C) 1999 Andy Adler
-##   This code has no warrany whatsoever.
-##   Do what you like with this code as long as you
-##   leave this copyright in place.
+## This program is free software; you can redistribute it and/or modify it under
+## the terms of the GNU General Public License as published by the Free Software
+## Foundation; either version 3 of the License, or (at your option) any later
+## version.
+##
+## This program is distributed in the hope that it will be useful, but WITHOUT
+## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+## FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+## details.
+##
+## You should have received a copy of the GNU General Public License along with
+## this program; if not, see <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {@var{bw} =} edge (@var{im}, @var{method})
 ## @deftypefnx{Function File} {@var{bw} =} edge (@var{im}, @var{method}, @var{arg1}, @var{arg2})
-## @deftypefnx{Function File} {[@var{bw}, @var{thresh}] =} edge (...)
+## @deftypefnx{Function File} {[@var{bw}, @var{thresh}] =} edge (@dots{})
 ## Detect edges in the given image using various methods. The first input @var{im}
 ## is the gray scale image in which edges are to be detected. The second argument
 ## controls which method is used for detecting the edges. The rest of the input
@@ -49,7 +42,7 @@
 ##
 ## @item "Prewitt"
 ## Finds the edges in @var{im} using the Prewitt approximation to the
-## derivatives. This method works just like "Sobel" except a different aproximation
+## derivatives. This method works just like "Sobel" except a different approximation
 ## the gradient is used.
 ##
 ## @item "Roberts"
@@ -130,11 +123,11 @@
 ## @item params(1)==0 or 4 or 8
 ## Perform x connected dilatation (step 3).
 ## @item params(2)
-## Dilatation coeficient (threshold) in step 3.
+## Dilatation coefficient (threshold) in step 3.
 ## @item params(3)
 ## Length of edge extention convolution (step 2).
 ## @item params(4)
-## Coeficient of extention convolution in step 2.
+## Coefficient of extention convolution in step 2.
 ## @end table
 ## defaults = [8, 1, 3, 3]
 ##
@@ -326,7 +319,7 @@ function [bw, out_threshold, g45_out, g135_out] = edge (im, method, varargin)
     #####################################
     case "zerocross"
       ## Get the filter
-      if (nargin == 4 && ismatrix(varargin{2}))
+      if (nargin == 4 && isnumeric (varargin{2}))
         f = varargin{2};
       else
         error("edge: a filter must be given as the fourth argument when 'zerocross' is used");
@@ -367,7 +360,7 @@ function [bw, out_threshold, g45_out, g135_out] = edge (im, method, varargin)
       ## Get thresholds
       if (nargin > 2 && isscalar(varargin{1}))
         thresh = [0.4*varargin{1}, varargin{1}];
-      elseif (nargin > 2 && ismatrix (varargin{1}) && length (varargin{1}(:)) == 2)
+      elseif (nargin > 2 && isnumeric (varargin{1}) && numel (varargin{1}) == 2)
         thresh = varargin{1}(:);
       else
         tmp = mean(abs(Es(:)));
@@ -432,7 +425,7 @@ function [bw, out_threshold, g45_out, g135_out] = edge (im, method, varargin)
   endif
 endfunction
 
-## An auxilary function that parses the 'direction' argument from 'varargin'
+## An auxiliary function that parses the 'direction' argument from 'varargin'
 function direction = get_direction(varargin)
   if (nargin >= 2)
     direction = varargin{2};
@@ -448,7 +441,7 @@ function direction = get_direction(varargin)
   endif
 endfunction
 
-## An auxilary function that performs a very simple thinning.
+## An auxiliary function that performs a very simple thinning.
 ## Strength is an image containing the edge strength.
 ## bw contains a 1 in (r,c) if
 ##  1) strength(r,c) is greater than both neighbours in the
@@ -465,7 +458,7 @@ function bw = simple_thinning(strength)
   bw = x | y;
 endfunction
 
-## Auxilary function. Finds the zero crossings of the 
+## Auxiliary function. Finds the zero crossings of the 
 ## 2-dimensional function f. (By Etienne Grossmann)
 function z = zerocrossings(f)
   z0 = f<0;                 ## Negative
@@ -481,7 +474,7 @@ endfunction
 
 ## The 'andy' edge detector that was present in older versions of 'edge'.
 ## The function body has simply been copied from the old implementation.
-##   -- Soren Hauberg, march 11th, 2008
+##   -- Søren Hauberg, march 11th, 2008
 function [imout, thresh] = andy(im, method, thresh, param2)
    [n,m]= size(im);
    xx= 2:m-1;
