@@ -1,22 +1,22 @@
-## Copyright (C) 2006  Sren Hauberg
-## 
-## This program is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2, or (at your option)
-## any later version.
-## 
-## This program is distributed in the hope that it will be useful, but
-## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details. 
-## 
-## You should have received a copy of the GNU General Public License
-## along with this file.  If not, see <http://www.gnu.org/licenses/>.
+## Copyright (C) 2006 Søren Hauberg <soren@hauberg.org>
+##
+## This program is free software; you can redistribute it and/or modify it under
+## the terms of the GNU General Public License as published by the Free Software
+## Foundation; either version 3 of the License, or (at your option) any later
+## version.
+##
+## This program is distributed in the hope that it will be useful, but WITHOUT
+## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+## FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+## details.
+##
+## You should have received a copy of the GNU General Public License along with
+## this program; if not, see <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} @var{warped} = imremap(@var{im}, @var{XI}, @var{YI})
 ## @deftypefnx{Function File} @var{warped} = imremap(@var{im}, @var{XI}, @var{YI}, @var{interp}, @var{extrapval})
-## @deftypefnx{Function File} [@var{warped}, @var{valid} ] = imremap(...)
+## @deftypefnx{Function File} [@var{warped}, @var{valid} ] = imremap(@dots{})
 ## Applies any geometric transformation to the image @var{im}.
 ##
 ## The arguments @var{XI} and @var{YI} are lookup tables that define the resulting
@@ -28,22 +28,16 @@
 ## by interpolation. Note that the image @var{im} is expressed in a (X, Y)-coordinate
 ## system and not a (row, column) system.
 ##
-## The argument @var{interp} selects the used interpolation method, and most be one
-## of the following strings
-## @table @code
-## @item "nearest"
-## Nearest neighbor interpolation.
-## @item "linear"
-## @itemx "bilinear"
-## Bilinear interpolation. This is the default behavior.
-## @item "cubic"
-## @itemx "bicubic"
-## Bicubic interpolation.
-## @end table
+## The optional argument @var{method} defines the interpolation method to be
+## used.  All methods supported by @code{interp2} can be used.  By default, the
+## @code{linear} method is used.
+##
+## For @sc{matlab} compatibility, the methods @code{bicubic} (same as
+## @code{cubic}), @code{bilinear} and @code{triangle} (both the same as
+## @code{linear}) are also supported.
 ##
 ## All values of the result that fall outside the original image will
-## be set to @var{extrapval}. For images of class @code{double} @var{extrapval}
-## defaults to @code{NA} and for other classes it defaults to 0.
+## be set to @var{extrapval}.  The default value of @var{extrapval} is 0.
 ##
 ## The optional output @var{valid} is a matrix of the same size as @var{warped}
 ## that contains the value 1 in pixels where @var{warped} contains an interpolated
@@ -51,59 +45,42 @@
 ## @seealso{imperspectivewarp, imrotate, imresize, imshear, interp2}
 ## @end deftypefn
 
-function [warped, valid] = imremap(im, XI, YI, interp = "bilinear", extrapval = NA)
-  ## Check input
-  if (nargin < 3)
-    print_usage();
-  endif
-  
-  [imrows, imcols, imchannels, tmp] = size(im);
-  if (tmp != 1 || (imchannels != 1 && imchannels != 3))
-    error("imremap: first input argument must be an image");
-  endif
-  
-  if (!size_equal(XI, YI) || !ismatrix(XI) || ndims(XI) != 2)
-    error("imremap: XI and YI must be matrices of the same size");
-  endif
-  
-  if (!any(strcmpi(interp, {"nearest", "linear", "bilinear", "cubic", "bicubic", "spline"})))
-    error("imremap: unsupported interpolation method");
-  endif
-  if (any(strcmpi(interp, {"bilinear", "bicubic"})))
-    interp = interp(3:end); # Remove "bi"
-  endif
-  interp = lower(interp);
-  
-  if (!isscalar(extrapval))
-    error("imremap: extrapolation value must be a scalar");
+function [warped, valid] = imremap(im, XI, YI, interp = "linear", extrapval = 0)
+
+  if (nargin < 3 || nargin > 5)
+    print_usage ();
+  elseif (! isimage (im) || ndims (im) > 3)
+    error ("imremap: IM must be a grayscale or RGB image.")
+  elseif (! size_equal (XI, YI) || ! ismatrix (XI) || ! isnumeric (XI))
+    error ("imremap: XI and YI must be matrices of the same size");
+  elseif (! ischar (interp))
+    error ("imremap: INTERP must be a string with interpolation method")
+  elseif (! isscalar (extrapval))
+    error ("imremap: EXTRAPVAL must be a scalar");
   endif
+  interp = interp_method (interp);
   
   ## Interpolate
-  if (imchannels == 1) # Gray
-    warped = grayinterp(im, XI, YI, interp, NA);
+  if (size (im, 3) == 1) # Gray
+    warped = grayinterp(im, XI, YI, interp, 0);
   else # rgb image
     for i = 3:-1:1
-      warped(:,:,i) = grayinterp(im(:,:,i), XI, YI, interp, NA);
+      warped(:,:,i) = grayinterp(im(:,:,i), XI, YI, interp, 0);
     endfor
   endif
   valid = !isna(warped);
   warped(!valid) = extrapval;
 
-  ## Change the class of the results according to the class of the image
-  c = class(im);
-  if (strcmpi(c, "uint8"))
-    warped = uint8(warped);
-  elseif (strcmpi(c, "uint16"))
-    warped = uint16(warped);
-  endif
+  ## we return image on same class as input
+  warped = cast (warped, class (im));
 
 endfunction
 
 function [warped, valid] = grayinterp(im, XI, YI, interp, extrapval)
   if (strcmp(interp, "cubic"))
-    warped = graybicubic(double(im), XI, YI, NA);
+    warped = graybicubic(double(im), XI, YI, 0);
   else
-    warped = interp2(double(im), XI, YI, interp, NA);
+    warped = interp2(double(im), XI, YI, interp, 0);
   endif
   valid = !isna(warped);
   warped(!valid) = extrapval;
@@ -119,7 +96,7 @@ endfunction
 ## @seealso{interp2}
 ## @end deftypefn
 
-function ZI = graybicubic (Z, XI, YI, extrapval = NA)
+function ZI = graybicubic (Z, XI, YI, extrapval = 0)
   
   ## Allocate output
   [X, Y] = meshgrid(1:columns(Z), 1:rows(Z));
