@@ -1,17 +1,17 @@
-// Copyright (C) 2008 Soren Hauberg
+// Copyright (C) 2008 Søren Hauberg <soren@hauberg.org>
 //
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 3
-// of the License, or (at your option) any later version.
+// This program is free software; you can redistribute it and/or modify it under
+// the terms of the GNU General Public License as published by the Free Software
+// Foundation; either version 3 of the License, or (at your option) any later
+// version.
 //
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
+// This program is distributed in the hope that it will be useful, but WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+// details.
 //
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, see <http://www.gnu.org/licenses/>.
+// You should have received a copy of the GNU General Public License along with
+// this program; if not, see <http://www.gnu.org/licenses/>.
 
 #include <octave/oct.h>
 
@@ -68,7 +68,7 @@ bilateral (const MatrixType &im, const double sigma_d, const double sigma_r)
           double k = 0;
           for (octave_idx_type i = 0; i < num_planes; i++)
             {
-              val[i] = im (r,c,i);
+              val[i] = im (r+s,c+s,i);
               sum[i] = 0;
             }
           for (octave_idx_type kr = 0; kr < 2*s+1; kr++)
